/* -*- mode: ObjC; tab-width: 8 -*-
   Copyright (C) 2003 TSUTSUMI Kikuo.
   This file is part of the CocoUnit Library.

   The CocoUnit Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public License
   as published by the Free Software Foundation; either version 2.1 of
   the License, or (at your option) any later version.

   The CocoUnit Library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the implied warranty
   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the CocoUnit Library; see the file COPYING.LESSER.
   If not, write to the Free Software Foundation, Inc., 59 Temple
   Place - Suite 330, Boston, MA 02111-1307, USA.  
*/
/*
 * $Id: CocoAssert.h,v 1.2 2003/12/06 02:15:14 tsutsumi Exp $
 */

/** @file
 */

#ifndef COCOASSERT_H
#define COCOASSERT_H

#include <CocoUnit/CocoTest.h>
#include <stdbool.h>

/**
 * A set of assert methods.
 */
@interface CocoAssert : CocoTest {

}

/**
 * ~english
 * Asserts that a condition is true. If it isn't it throws an
 * AssertionFailedError.
 * ~japanese
 * 郎Ǥ뤳Ȥɽޤ⤷Ǥʤ
 * AssertionFailedError ꤲޤ
 */
-(void) assertTrue:(bool) condition;
-(void) assertTrue:(NSString*) message condition:(bool) condition;
-(void) assertTrue:(bool) condition format:(NSString*) format, ...;

/**
 * Asserts that a condition is false. If it isn't it throws
 * an AssertionFailedError.
 */
-(void) assertFalse:(bool) condition;
-(void) assertFalse:(NSString*) message condition:(bool) condition;
-(void) assertFalse:(bool) condition format:(NSString*) format, ...;

/**
 * ~english
 * Asserts that two objects are equal. Examine the matter whether
 * their are equal in the "isEqual:" method. If they are not an
 * AssertionFailedError is thrown.
 *
 * ~japanese
 * ĤΥ֥ȤƤȤɽޤ
 * ɤ isEqual: ᥽åɤĴ٤ޤ
 * ⤷Ǥʤ AssertionFailedError ꤲޤ
 *
 * ~
 *
 * @param message the detail message for this assertion
 * @param expected the expected value of an object
 * @param actual the actual value of an object
 * @param format the detail message for this assertion
 */
-(void) assertEquals:(NSObject*) expected actual:(NSObject*) actual;
-(void) assertEquals:(NSString*) message
	    expected:(NSObject*) expected
	      actual:(NSObject*) actual;
-(void) assertEquals:(NSObject*) expected
	      actual:(NSObject*) actual
	      format:(NSString*) format, ...;

/**
 * Asserts that two Strings are equal by using the isEqualString:
 * method. When both objects are strings, isEqualString: method is a
 * faster way to check equality than isEqual:.
 */
-(void) assertEqualsString:(NSString*) expected actual:(NSString*) actual;
-(void) assertEqualsString:(NSString*) message
		  expected:(NSString*) expected
		    actual:(NSString*) actual;
-(void) assertEqualsString:(NSString*) expected
		    actual:(NSString*) actual
		    format:(NSString*) format, ...;

/**
 * Asserts that two doubles are equal.
 * @param message the detail message for this assertion
 * @param expected the expected value of an object
 * @param actual the actual value of an object
 * @param delta tolerated delta
 * @param format the detail message format for this assertion
 */
-(void) assertEquals:(double) expected
	      actual:(double) actual
	       delta:(double) delta;
-(void) assertEquals:(NSString*) message
	    expected:(double) expected
	      actual:(double) actual
	       delta:(double) delta;
-(void) assertEquals:(double) expected
	      actual:(double) actual
	       delta:(double) delta
	      format:(NSString*) format, ...;
-(void) assertEqualsDouble:(double) expected
		    actual:(double) actual
		     delta:(double) delta;
-(void) assertEqualsDouble:(NSString*) message
		  expected:(double) expected
		    actual:(double) actual
		     delta:(double) delta;
-(void) assertEqualsDouble:(double) expected
		    actual:(double) actual
		     delta:(double) delta
		    format:(NSString*) format, ...;

/**
 * Asserts that two floats are equal concerning a delta. If they are
 * not an AssertionFailedError is thrown.  If the expected value is
 * infinity then the delta value is ignored.
 */
-(void) assertEqualsFloat:(float) expected
		   actual:(float) actual
		    delta:(float) delta;
-(void) assertEqualsFloat:(NSString*) message
		 expected:(float) expected
		   actual:(float) actual
		    delta:(float) delta;
-(void) assertEqualsFloat:(float) expected
		   actual:(float) actual
		    delta:(float) delta
		   format:(NSString*) format, ...;

/**
 * Asserts that two longs are equal.
 * @param message the detail message for this assertion
 * @param expected the expected value of an object
 * @param actual the actual value of an object
 * @param format the detail message for this assertion
 */
-(void) assertEqualsLong:(long) expected actual:(long) actual;
-(void) assertEqualsLong:(NSString*) message
		expected:(long) expected
		  actual:(long) actual;
-(void) assertEqualsLong:(long) expected
		  actual:(long) actual
		  format:(NSString*) format, ...;

/**
 * Asserts that two booleans are equal. If they are not
 * an AssertionFailedError is thrown.
 */
-(void) assertEqualsBool:(bool) expected actual:(bool) actual;
-(void) assertEqualsBool:(NSString*) message
		expected:(bool) expected
		  actual:(bool) actual;
-(void) assertEqualsBool:(bool) expected
		  actual:(bool) actual
		  format:(NSString*) format, ...;

/**
 * Asserts that two bytes are equal. If they are not
 * an AssertionFailedError is thrown.
 */
-(void) assertEqualsUnsignedChar:(unsigned char) expected
			  actual:(unsigned char) actual;
-(void) assertEqualsUnsignedChar:(NSString*) message
			expected:(unsigned char) expected
			  actual:(unsigned char) actual;
-(void) assertEqualsUnsignedChar:(unsigned char) expected
			  actual:(unsigned char) actual
			  format:(NSString*) format, ...;

/**
 * Asserts that two chars are equal. If they are not
 * an AssertionFailedError is thrown.
 */
-(void) assertEqualsChar:(char) expected actual:(char) actual;
-(void) assertEqualsChar:(NSString*) message
		expected:(char) expected
		  actual:(char) actual;
-(void) assertEqualsChar:(char) expected
		  actual:(char) actual
		  format:(NSString*) format, ...;

/**
 * Asserts that two shorts are equal. If they are not
 * an AssertionFailedError is thrown.
 */
-(void) assertEqualsShort:(short) expected actual:(short) actual;
-(void) assertEqualsShort:(NSString*) message
		 expected:(short) expected
		   actual:(short) actual;
-(void) assertEqualsShort:(short) expected
		   actual:(short) actual
		   format:(NSString*) format, ...;

/**
 * Asserts that two ints are equal. If they are not
 * an AssertionFailedError is thrown.
 */
-(void) assertEqualsInt:(int) expected actual:(int) actual;
-(void) assertEqualsInt:(NSString*) message
	       expected:(int) expected
		 actual:(int) actual;
-(void) assertEqualsInt:(int) expected
		 actual:(int) actual
		 format:(NSString*) format, ...;

/**
 * ~english
 * Asserts that an object isn't null.
 * ~japanese
 * ֥Ȥ nil ǤʤȤɽޤ
 */
-(void) assertNotNil:(id) object;
-(void) assertNotNil:(NSString*) message actual:(id) object;
-(void) assertNotNil:(id) object format:(NSString*) format, ...;

/**
 * ~english
 * Asserts that an object is nil.
 * ~japanese
 * ֥Ȥ nil Ǥ뤳Ȥɽޤ
 */
-(void) assertNil:(id) object;
-(void) assertNil:(NSString*) message actual:(id) object;
-(void) assertNil:(id) object format:(NSString*) format, ...;

/**
 * ~english
 * Asserts that two objects refer to the same object. If they are not
 * the same an AssertionFailedError is thrown.
 * ~japanese
 * ĤΥ֥ȤƱ֥Ȥ򻲾ȤƤ뤳Ȥɽޤ
 * ⤷Ǥʤ AssertionFailedError ꤲޤ
 * ~
 * @param expected the expected value of an object
 * @param actual the actual value of an object
 * @param message the detail message for this assertion
 * @param format the detail message for this assertion
 */
-(void) assertSame:(id) expected actual:(id) actual;
-(void) assertSame:(NSString*) message expected:(id) expected actual:(id) actual;
-(void) assertSame:(id) expected actual:(id) actual format:(NSString*) format,...;

/**
 * Asserts that two objects refer to the same object. If they are not
 * an AssertionFailedError is thrown with the given message.
 */
-(void) assertNotSame: expected actual: actual;
-(void) assertNotSame:(NSString*) message expected: expected actual: actual;
-(void) assertNotSame:(id) expected actual:(id) actual format:(NSString*) format,...;

/**
 * Fails a test. 
 */
-(void) fail;
-(void) fail:(NSString*) reason;
-(void) failWithFormat:(NSString*) format, ...;

/**
 * Error a test.
 */
-(void) error;
-(void) error:(NSString*) reason;
-(void) errorWithFormat:(NSString*) format, ...;

-(void) failSame:(NSString*) message;
-(void) failSameWithFormat:(NSString*) format, ...;

/**
 * Fails a test with the given message and values.
 */
-(void) failNotSame:(NSString*) message expected:(id) expected actual:(id) actual;
-(void) failNotSame:(id) expected actual:(id) actual format:(NSString*) format,...;

/**
 * Fails a test with the given message and values.
 */
-(void) failNotEquals:(NSString*) message expected:(id) expected actual:(id) actual;
-(void) failNotEquals:(id) expected actual:(id) actual format:(NSString*) format,...;

+(NSString*) format:(NSString*) message expected:(id) expected actual:(id) actual;
+(NSString*) format:(id) expected actual:(id) actual format:(NSString*) format, ...;

@end
#endif // COCOASSERT_H
// Local variables: ***
// compile-command: "xcodebuild -project CocoUnit.xcode -target CocoUnitTest" ***
// End: ***
