/* -*- ObjC -*-
   Copyright (C) 2003 TSUTSUMI Kikuo.
   This file is part of the CocoUnit Library.

   The CocoUnit Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public License
   as published by the Free Software Foundation; either version 2.1 of
   the License, or (at your option) any later version.

   The CocoUnit Library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the implied warranty
   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the CocoUnit Library; see the file COPYING.LESSER.
   If not, write to the Free Software Foundation, Inc., 59 Temple
   Place - Suite 330, Boston, MA 02111-1307, USA.  
*/
/*
 * $Id: CocoAssert.m,v 1.2 2003/12/06 02:15:14 tsutsumi Exp $
 */

/** @file
 */

#import <Foundation/NSString.h>
#include <CocoUnit/CocoAssert.h>
#include <CocoUnit/CocoAssertionFailedError.h>
#include <CocoUnit/CocoComparisonFailure.h>
#include <CocoUnit/CocoError.h>
#include <stdarg.h>

#define FORMAT_MESSAGE(message)						\
  do {									\
    if (!format || ![ format length ])					\
      message = nil;							\
    else								\
      {									\
	va_list args;							\
	va_start(args, format);						\
	message = [ [ NSString alloc ] initWithFormat: format arguments: args ]; \
	va_end(args);							\
	[ message autorelease ];					\
      }									\
  } while (0)

@implementation CocoAssert

-(void) assertTrue:(bool) condition
{
  [ self assertTrue: nil condition: condition ];
}

-(void) assertTrue:(NSString*) message condition:(bool) condition
{
  if (condition)
    return;
  [ self fail: message ];
}

-(void) assertTrue:(bool) condition format:(NSString*) format, ...
{
  NSString* message;
  FORMAT_MESSAGE(message);
  [ self assertTrue: message condition: condition ];
}

-(void) assertFalse:(bool) condition
{
  [ self assertFalse: nil condition: condition ];
}

-(void) assertFalse:(NSString*) message condition:(bool) condition
{
  [ self assertTrue: message condition: !condition ];
}

-(void) assertFalse:(bool) condition format:(NSString*) format, ...
{
  NSString* message;
  FORMAT_MESSAGE(message);
  [ self assertTrue: message condition: !condition ];
}

-(void) assertEquals:(NSObject*) expected actual:(NSObject*) actual
{
  [ self assertEquals: nil expected: expected actual: actual ];
}

-(void) assertEquals:(NSString*) message
	    expected:(NSObject*) expected actual:(NSObject*) actual
{
  if (expected == nil && actual == nil)
    return;
  if (expected != nil
      && (([ expected isKindOfClass: [ NSString class ] ]
           && [ actual isKindOfClass: [ NSString class ] ])
	  ? [ (NSString*)expected isEqualToString: (NSString*)actual ]
	  : [ expected isEqual: actual ]))
    return;
  [ self failNotEquals: message expected: expected actual: actual ];
}

-(void) assertEquals:(NSObject*) expected actual:(NSObject*) actual
	      format:(NSString*) format, ...
{
  NSString* message;
  FORMAT_MESSAGE(message);
  [ self assertEquals: message expected: expected actual: actual ];
}

-(void) assertEqualsString:(NSString*) expected actual:(NSString*) actual
{
  [ self assertEqualsString: nil expected: expected actual: actual ];
}

-(void) assertEqualsString:(NSString*) message
		  expected:(NSString*) expected
		    actual:(NSString*) actual
{
  if (expected == nil && actual == nil)
    return;
  if (expected != nil && [ expected isEqualToString: actual ])
    return;
  [ CocoComparisonFailure raise: message expected: expected actual: actual ];
}

-(void) assertEqualsString:(NSString*) expected
		    actual:(NSString*) actual
		    format:(NSString*) format, ...
{
  NSString* message;
  FORMAT_MESSAGE(message);
  [ self assertEqualsString: message expected: expected actual: actual ];
}

-(void) assertEquals:(double) expected
	      actual:(double) actual
	       delta:(double) delta
{
  [ self assertEqualsDouble: nil expected: expected actual: actual delta: delta ];
}

-(void) assertEquals:(NSString*) message
	    expected:(double) expected
	      actual:(double) actual
	       delta:(double) delta
{
  [ self assertEqualsDouble: message expected: expected actual: actual delta: delta];
}

-(void) assertEquals:(double) expected
	      actual:(double) actual
	       delta:(double) delta
	      format:(NSString*) format, ...
{
  NSString* message;
  FORMAT_MESSAGE(message);
  [ self assertEqualsDouble: message expected: expected actual: actual delta: delta];
}

-(void) assertEqualsDouble:(double) expected
		    actual:(double) actual
		     delta:(double) delta
{
  [ self assertEqualsDouble: nil expected: expected actual: actual delta: delta ];
}

-(void) assertEqualsDouble:(NSString*) message
		  expected:(double) expected
		    actual:(double) actual
		     delta:(double) delta
{
  // handle infinity specially since subtracting to infinite values
  // gives NaN and the following test fails
  if (isinf (expected))
    {
      if (expected == actual)
	return;
    }
  else if (fabs (expected - actual) <= delta)
    return;
  // Because comparision with NaN always returns false
  [ self failNotEquals: message
	 expected: [ NSNumber numberWithDouble: expected ]
	 actual: [ NSNumber numberWithDouble: actual ] ];
}

-(void) assertEqualsDouble:(double) expected
		    actual:(double) actual
		     delta:(double) delta
		    format:(NSString*) format, ...
{
  NSString* message;
  FORMAT_MESSAGE(message);
  [ self assertEqualsDouble: message expected: expected actual: actual delta: delta ];
}

-(void) assertEqualsFloat:(float) expected
		   actual:(float) actual
		    delta:(float) delta
{
  [ self assertEqualsFloat: nil expected: expected actual: actual delta: delta ];
}

-(void) assertEqualsFloat:(NSString*) message
		 expected:(float) expected
		   actual:(float) actual
		    delta:(float) delta
{
  // handle infinity specially since subtracting to infinite values
  // gives NaN and the following test fails
  if (isinf (expected))
    {
      if (expected == actual)
	return;
    }
  else if (fabsf (expected - actual) <= delta)
    return;
  [ self failNotEquals: message
	 expected: [ NSNumber numberWithFloat: expected ]
	 actual: [ NSNumber numberWithFloat: actual ] ];
}

-(void) assertEqualsFloat:(float) expected
		   actual:(float) actual
		    delta:(float) delta
		   format:(NSString*) format, ...
{
  NSString* message;
  FORMAT_MESSAGE(message);
  [ self assertEqualsFloat: message expected: expected actual: actual delta: delta ];
}

-(void) assertEqualsLong:(long) expected actual:(long) actual
{
  [ self assertEqualsLong: expected actual: actual format: nil ];
}

-(void) assertEqualsLong:(NSString*) message
		expected:(long) expected
		  actual:(long) actual
{
  [ self assertEquals: message
	 expected: [ NSNumber numberWithLong: expected ]
	 actual: [ NSNumber numberWithLong: actual ] ];
}

-(void) assertEqualsLong:(long) expected
		  actual:(long) actual
		  format:(NSString*) format, ...
{
  NSString* message;
  FORMAT_MESSAGE(message);
  [ self assertEqualsLong: message expected: expected actual: actual ];
}

-(void) assertEqualsBool:(bool) expected
		  actual:(bool) actual
{
  [ self assertEqualsBool: nil expected: expected actual: actual ];
}

-(void) assertEqualsBool:(NSString*) message
		expected:(bool) expected
		  actual:(bool) actual
{
  [ self assertEquals: message
	 expected: [ NSNumber numberWithBool: expected ]
	 actual: [ NSNumber numberWithBool: actual ] ];
}

-(void) assertEqualsBool:(bool) expected
		  actual:(bool) actual
		  format:(NSString*) format, ...
{
  NSString* message;
  FORMAT_MESSAGE(message);
  [ self assertEqualsBool: message expected: expected actual: actual ];
}

-(void) assertEqualsUnsignedChar:(unsigned char) expected
			  actual:(unsigned char) actual
{
  [ self assertEqualsUnsignedChar: expected actual: actual ];
}

-(void) assertEqualsUnsignedChar:(NSString*) message
			expected:(unsigned char) expected
			  actual:(unsigned char) actual
{
  [ self assertEquals: message
	 expected: [ NSNumber numberWithUnsignedChar: expected ]
	 actual: [ NSNumber numberWithUnsignedChar: actual ] ];
}

-(void) assertEqualsUnsignedChar:(unsigned char) expected
			  actual:(unsigned char) actual
			  format:(NSString*) format, ...
{
  NSString* message;
  FORMAT_MESSAGE(message);
  [ self assertEqualsUnsignedChar: message expected: expected actual: actual ];
}

-(void) assertEqualsChar:(char) expected
		  actual:(char) actual
{
  [ self assertEqualsChar: nil expected: expected actual: actual ];
}

-(void) assertEqualsChar:(NSString*) message
		expected:(char) expected
		  actual:(char) actual
{
  [ self assertEquals: message
	 expected: [ NSNumber numberWithChar: expected ]
	 actual: [ NSNumber numberWithChar: actual ] ];
}

-(void) assertEqualsChar:(char) expected
		  actual:(char) actual
		  format:(NSString*) format, ...
{
  NSString* message;
  FORMAT_MESSAGE(message);
  [ self assertEqualsChar: message expected: expected actual: actual ];
}

-(void) assertEqualsShort:(short) expected
		   actual:(short) actual
{
  [ self assertEqualsShort: nil expected: expected actual: actual ];
}

-(void) assertEqualsShort:(NSString*) message
		 expected:(short) expected
		   actual:(short) actual
{
  [ self assertEquals: message
	 expected: [ NSNumber numberWithShort: expected ]
	 actual: [ NSNumber numberWithShort: actual ] ];
}

-(void) assertEqualsShort:(short) expected
		   actual:(short) actual
		   format:(NSString*) format, ...
{
  NSString* message;
  FORMAT_MESSAGE(message);
  [ self assertEqualsShort: message expected: expected actual: actual ];
}

-(void) assertEqualsInt:(int) expected
		 actual:(int) actual
{
  [ self assertEqualsInt: expected actual: actual format: nil ];
}

-(void) assertEqualsInt:(NSString*) message
	       expected:(int) expected
		 actual:(int) actual
{
  [ self assertEquals: message
	 expected: [ NSNumber numberWithInt: expected ]
	 actual: [ NSNumber numberWithInt: actual ] ];
}

-(void) assertEqualsInt:(int) expected
		 actual:(int) actual
		 format:(NSString*) format, ...
{
  NSString* message;
  FORMAT_MESSAGE(message);
  [ self assertEqualsInt: message expected: expected actual: actual ];
}

-(void) assertNotNil:(id) object
{
  [ self assertNotNil: object format: nil ];
}

-(void) assertNotNil:(NSString*) message actual:(id) object
{
  [ self assertTrue: message condition: object != nil ];
}

-(void) assertNotNil:(id) object format:(NSString*) format, ...
{
  NSString* message;
  FORMAT_MESSAGE(message);
  [ self assertNotNil: message actual: object ];
}

-(void) assertNil:(id) object
{
  [ self assertNil: nil actual: object ];
}

-(void) assertNil:(NSString*) message actual:(id) object
{
  [ self assertTrue: message condition: object == nil ];
}

-(void) assertNil:(id) object format:(NSString*) format, ...
{
  NSString* message;
  FORMAT_MESSAGE(message);
  [ self assertNil: message actual: object ];
}

-(void) assertSame:(id) expected actual:(id) actual
{
  [ self assertSame: nil expected: expected actual: actual ];
}

-(void) assertSame:(NSString*) message expected:(id) expected actual:(id) actual
{
  if (expected == actual)
    return;
  [ self failNotSame: message expected: expected actual: actual ];
}

-(void) assertSame:(id) expected actual:(id) actual format:(NSString*) format, ...
{
  NSString* message;
  FORMAT_MESSAGE(message);
  [ self assertSame: message expected: expected actual: actual ];
}

-(void) assertNotSame: expected actual: actual
{
  [ self assertNotSame: nil expected: expected actual: actual ];
}

-(void) assertNotSame:(NSString*) message expected:(id) expected actual:(id) actual
{
  if (expected != actual)
    return;
  [ self failSame: message ];
}

-(void) assertNotSame:(id) expected actual:(id) actual format:(NSString*) format, ...
{
  NSString* message;
  FORMAT_MESSAGE(message);
  [ self assertNotSame: message expected: expected actual: actual ];
}

-(void) fail
{
  [ self fail: nil ];
}

-(void) fail:(NSString*) reason
{
  [ CocoAssertionFailedError raise: reason ];
}

-(void) failWithFormat:(NSString*) format, ...
{
  NSString* reason;
  FORMAT_MESSAGE(reason);
  [ self fail: reason ];
}

-(void) error
{
  [ self error: nil ];
}

-(void) error:(NSString*) reason
{
  [ CocoError raise: reason ];
}

-(void) errorWithFormat:(NSString*) format, ...
{
  NSString* reason;
  FORMAT_MESSAGE(reason);
  [ self error: reason ];
}

-(void) failSame:(NSString*) message
{
  NSString* separator = (!message ? (message = @"")
			 : ![ message length ] ? @"" : @" ");
  [ self failWithFormat: @"%@%@expected not same", message, separator ];
}

-(void) failSameWithFormat:(NSString*) format, ...
{
  NSString* message;
  FORMAT_MESSAGE(message);
  [ self failSame: message ];
}

-(void) failNotSame:(NSString*) message expected:(id) expected actual:(id) actual
{
  NSString* separator = (!message ? (message = @"")
			 : ![ message length ] ? @"" : @" ");
  [ self failWithFormat: @"%@%@expected same:<%@> but was <%@>",
	 message, separator, expected, actual ];
}

-(void) failNotSame:(id) expected actual:(id) actual format:(NSString*) format, ...
{
  NSString* message;
  FORMAT_MESSAGE(message);
  [ self failNotSame: message expected: expected actual: actual ];
}

-(void) failNotEquals:(NSString*) message
	     expected:(id) expected
	       actual:(id) actual
{
  NSString* reason;
  reason = [ CocoAssert format: message expected: expected actual: actual ];
  [ self fail: reason ];
}

-(void) failNotEquals:(id) expected
	       actual:(id) actual
	       format:(NSString*) format, ...
{
  NSString* message;
  FORMAT_MESSAGE(message);
  [ self failNotEquals: message expected: expected actual: actual ];
}

+(NSString*) format:(NSString*) message
	   expected:(id) expected
	     actual:(id) actual
{
  NSString* separator = (!message ? (message = @"")
			 : ![ message length ] ? @"" : @" ");
  return [ NSString stringWithFormat: @"%@%@expected:<%@> but was:<%@>",
		    message, separator, expected, actual ];
}

+(NSString*) format:(id) expected
	     actual:(id) actual
	     format:(NSString*) format, ...
{
  NSString* message;
  FORMAT_MESSAGE(message);
  return [ self format: message expected: expected actual: actual ];
}
@end
// Local variables: ***
// compile-command: "xcodebuild -project CocoUnit.xcode -target CocoUnitTest" ***
// End: ***
