/* -*- ObjC -*-
   Copyright (C) 2003 TSUTSUMI Kikuo.
   This file is part of the CocoUnit Library.

   The CocoUnit Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public License
   as published by the Free Software Foundation; either version 2.1 of
   the License, or (at your option) any later version.

   The CocoUnit Library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the implied warranty
   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the CocoUnit Library; see the file COPYING.LESSER.
   If not, write to the Free Software Foundation, Inc., 59 Temple
   Place - Suite 330, Boston, MA 02111-1307, USA.  
*/
/*
 * $Id: CocoTestCase.h,v 1.2 2003/12/06 02:15:14 tsutsumi Exp $
 */

/** @file
 */

#ifndef COCOTESTCASE_H
#define COCOTESTCASE_H

#include <CocoUnit/CocoAssert.h>

@class CocoTestResult;

/**
 * A test case defines the fixture to run multiple tests. To define a test case<br>
 * 1) implement a subclass of CocoTestCase<br>
 * 2) define instance variables that store the state of the fixture<br>
 * 3) initialize the fixture state by overriding <code>setUp</code><br>
 * 4) clean-up after a test by overriding <code>tearDown</code>.<br>
 * Each test runs in its own fixture so there
 * can be no side effects among test runs.
 * Here is an example:
 * <pre>
 * public class MathTest extends CocoTestCase {
 *     protected double fValue1;
 *     protected double fValue2;
 *
 *     public MathTest(String name) {
 *         super(name);
 *     }
 *
 *    protected void setUp() {
 *         fValue1= 2.0;
 *         fValue2= 3.0;
 *     }
 * }
 * </pre>
 *
 * For each test implement a method which interacts
 * with the fixture. Verify the expected results with assertions specified
 * by calling <code>assert</code> with a boolean.
 * <pre>
 *    protected void testAdd() {
 *        double result= fValue1 + fValue2;
 *        assert(result == 5.0);
 *    }
 * </pre>
 * Once the methods are defined you can run them. The framework supports
 * both a static type safe and more dynamic way to run a test.
 * In the static way you override the runTest method and define the method to
 * be invoked. A convenient way to do so is with an anonymous inner class.
 * <pre>
 * CocoTest test= new MathTest("add") {
 *        public void runTest() {
 *            testAdd();
 *        }
 * };
 * test.run();
 * </pre>
 * The dynamic way uses reflection to implement <code>runTest</code>. It dynamically finds
 * and invokes a method.
 * In this case the name of the test case has to correspond to the test method
 * to be run.
 * <pre>
 * CocoTest= new MathTest("testAdd");
 * test.run();
 * </pre>
 * The tests to be run can be collected into a CocoTestSuite. JUnit provides
 * different <i>test runners</i> which can run a test suite and collect the results.
 * A test runner either expects a static method <code>suite</code> as the entry
 * point to get a test to run or it will extract the suite automatically.
 * <pre>
 * public static CocoTest suite() {
 *      suite.addTest(new MathTest("testAdd"));
 *      suite.addTest(new MathTest("testDivideByZero"));
 *      return suite;
 *  }
 * </pre>
 * @see CocoTestResult
 * @see CocoTestSuite
 */

@interface CocoTestCase : CocoAssert {
  /**
   * the name of the test case
   */
  NSString* name;
}

+(id) suite;

/**
 * Construct a test case.
 */
+(id) testCase;

/**
 * Construct a test case with the given name.
 */
+(id) testCaseWithName:(NSString*) caseName;

/**
 * Initialize a test case with the given name.
 */
-(id) initWithName:(NSString*) caseName;

/**
 * Destruct an object.
 */
-(void)dealloc;

/**
 * Counts the number of test cases executed by run(TestResult result).
 */
-(int) countTestCases;

/**
 * Gets the name of the test case.
 */
-(NSString*) name;

/**
 * A convenience method to run this test, collecting the results with a
 * default TestResult object.
 *
 * @see TestResult
 */
-(CocoTestResult*) run;

/**
 * Runs the test case and collects the results in TestResult.
 */
-(void) runWithResult:(CocoTestResult*) result;

/**
 * Runs the bare test sequence.
 * @exception Throwable if any exception is thrown
 */
-(void) runBare;

/**
 * Override to run the test and assert its state.
 * @exception Throwable if any exception is thrown
 */
-(void) runTest;

/**
 * Sets up the fixture, for example, open a network connection.
 * This method is called before a test is executed.
 */
-(void) setUp;

/**
 * Tears down the fixture, for example, close a network connection.
 * This method is called after a test is executed.
 */
-(void) tearDown;

/**
 * Returns a string representation of the test case
 */
-(NSString*) description;
@end

#endif // COCOTESTCASE_H
