/* -*- ObjC -*-
   Copyright (C) 2003 TSUTSUMI Kikuo.
   This file is part of the CocoUnit Library.

   The CocoUnit Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public License
   as published by the Free Software Foundation; either version 2.1 of
   the License, or (at your option) any later version.

   The CocoUnit Library is distributed in the hope that it will be
   useful, but WITHOUT ANY WARRANTY; without even the implied warranty
   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the CocoUnit Library; see the file COPYING.LESSER.
   If not, write to the Free Software Foundation, Inc., 59 Temple
   Place - Suite 330, Boston, MA 02111-1307, USA.  
*/
/*
 * $Id: CocoTestResult.h,v 1.1 2003/11/29 12:34:30 tsutsumi Exp $
 */

/** @file
 */

#ifndef COCOTESTRESULT_H
#define COCOTESTRESULT_H

#import <Foundation/NSObject.h>
#include <stdbool.h>

@class CocoTest;
@class CocoTestCase;
@class CocoTestListener;
@class CocoAssertionFailedError;
@class CocoProtectable;

/**
 * A <code>CocoTestResult</code> collects the results of executing
 * a test case. It is an instance of the Collecting Parameter pattern.
 * The test framework distinguishes between <i>failures</i> and <i>errors</i>.
 * A failure is anticipated and checked for with assertions. Errors are
 * unanticipated problems like an <code>ArrayIndexOutOfBoundsException</code>.
 *
 * @see CocoTest
 */

@interface CocoTestResult : NSObject {
  NSMutableArray* failures;
  NSMutableArray* errors;
  NSMutableArray* listeners;
  int runTests;
 @private
  bool stop;
}

+(id) testResult;

-(id) init;

/**
 * Adds an error to the list of errors. The passed in exception
 * caused the error.
 */
-(void) addTest:(CocoTest*) test error:(NSException*) error;

/**
 * Adds a failure to the list of failures. The passed in exception
 * caused the failure.
 */
-(void) addTest:(CocoTest*) test failure:(CocoAssertionFailedError*) failure;

/**
 * Registers a CocoTestListener
 */
-(void) addListener:(CocoTestListener*) aListener;

/**
 * Deregister a CocoTestListener
 */
-(void) removeListener:(CocoTestListener*) aListener;

/**
 * Informs the result that a test was completed.
 */
-(void) endTest:(CocoTest*) test;

/**
 * Gets the number of detected errors.
 */
-(unsigned int) errorCount;

/**
 * Returns an Enumeration for the errors
 */
-(NSEnumerator*) errors;

/**
 * Gets the number of detected errors.
 */
-(unsigned int) failureCount;

/**
 * Returns an Enumeration for the failures
 */
-(NSEnumerator*) failures;

/**
 * Runs a CocoTestCase.
 */
-(void) runWithTest:(CocoTestCase*) test;

/**
 * Gets the number of run tests.
 */
-(int) runCount;

-(void) runProtected:(CocoProtectable*) p;

/**
 * Checks whether the test run should stop
 */
-(bool) shouldStop;

/**
 * Informs the result that a test will be started.
 */
-(void) startTest:(CocoTest*) test;

/**
 * Marks that the test run should stop.
 */
-(void) stop;

/**
 * Returns whether the entire test was successful or not.
 */
-(bool) wasSuccessful;

@end

#endif // COCOTESTRESULT_H
